import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def apply_pose_asset(override_context: typing.
                     Union[typing.Dict, 'bpy.types.Context'] = None,
                     execution_context: typing.Union[str, int] = None,
                     undo: bool = None,
                     *,
                     blend_factor: float = 1.0,
                     flipped: bool = False):
    ''' Apply the given Pose Action to the rig

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    :param blend_factor: Blend Factor, Amount that the pose is applied on top of the existing poses
    :type blend_factor: float
    :param flipped: Apply Flipped, When enabled, applies the pose flipped over the X-axis
    :type flipped: bool
    '''

    pass


def apply_pose_asset_for_keymap(
        override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: bool = None):
    ''' Apply the given Pose Action to the rig :file: `addons/pose_library/operators.py\:438 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$438>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def blend_pose_asset(override_context: typing.
                     Union[typing.Dict, 'bpy.types.Context'] = None,
                     execution_context: typing.Union[str, int] = None,
                     undo: bool = None,
                     *,
                     blend_factor: float = 0.0,
                     flipped: bool = False,
                     release_confirm: bool = False):
    ''' Blend the given Pose Action to the rig

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    :param blend_factor: Blend Factor, Amount that the pose is applied on top of the existing poses
    :type blend_factor: float
    :param flipped: Apply Flipped, When enabled, applies the pose flipped over the X-axis
    :type flipped: bool
    :param release_confirm: Confirm on Release, Always confirm operation when releasing button
    :type release_confirm: bool
    '''

    pass


def blend_pose_asset_for_keymap(
        override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: bool = None):
    ''' Blend the given Pose Action to the rig :file: `addons/pose_library/operators.py\:406 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$406>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def convert_old_object_poselib(
        override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: bool = None):
    ''' Create a pose asset for each pose marker in this legacy pose library data-block :file: `addons/pose_library/operators.py\:494 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$494>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def convert_old_poselib(override_context: typing.
                        Union[typing.Dict, 'bpy.types.Context'] = None,
                        execution_context: typing.Union[str, int] = None,
                        undo: bool = None):
    ''' Create a pose asset for each pose marker in the current action :file: `addons/pose_library/operators.py\:460 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$460>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def copy_as_asset(override_context: typing.
                  Union[typing.Dict, 'bpy.types.Context'] = None,
                  execution_context: typing.Union[str, int] = None,
                  undo: bool = None):
    ''' Create a new pose asset on the clipboard, to be pasted into an Asset Browser :file: `addons/pose_library/operators.py\:211 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$211>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def create_pose_asset(override_context: typing.
                      Union[typing.Dict, 'bpy.types.Context'] = None,
                      execution_context: typing.Union[str, int] = None,
                      undo: bool = None,
                      *,
                      pose_name: str = "",
                      activate_new_action: bool = True):
    ''' Create a new Action that contains the pose of the selected bones, and mark it as Asset. The asset will be stored in the current blend file

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    :param pose_name: Pose Name
    :type pose_name: str
    :param activate_new_action: Activate New Action
    :type activate_new_action: bool
    '''

    pass


def paste_asset(override_context: typing.
                Union[typing.Dict, 'bpy.types.Context'] = None,
                execution_context: typing.Union[str, int] = None,
                undo: bool = None):
    ''' Paste the Asset that was previously copied using Copy As Asset :file: `addons/pose_library/operators.py\:286 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$286>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass


def pose_asset_select_bones(override_context: typing.
                            Union[typing.Dict, 'bpy.types.Context'] = None,
                            execution_context: typing.Union[str, int] = None,
                            undo: bool = None,
                            *,
                            select: bool = True,
                            flipped: bool = False):
    ''' Select those bones that are used in this pose

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    :param select: Select
    :type select: bool
    :param flipped: Flipped
    :type flipped: bool
    '''

    pass


def restore_previous_action(override_context: typing.
                            Union[typing.Dict, 'bpy.types.Context'] = None,
                            execution_context: typing.Union[str, int] = None,
                            undo: bool = None):
    ''' Switch back to the previous Action, after creating a pose asset :file: `addons/pose_library/operators.py\:164 <https://developer.blender.org/diffusion/BA/addons/pose_library/operators.py$164>`_

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    '''

    pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


class KDTree:
    ''' KdTree(size) -> new kd-tree initialized to hold ``size`` items.
    '''

    def balance(self):
        ''' Balance the tree.

        '''
        pass

    def find(self, co: typing.Tuple[float],
             filter: typing.Callable = None) -> typing.Tuple:
        ''' Find nearest point to ``co``.

        :param co: 3d coordinates.
        :type co: typing.Tuple[float]
        :param filter: function which takes an index and returns True for indices to include in the search.
        :type filter: typing.Callable
        :rtype: typing.Tuple
        :return: `Vector`, index, distance).
        '''
        pass

    def find_n(self, co: typing.Tuple[float], n: int) -> typing.List:
        ''' Find nearest ``n`` points to ``co``.

        :param co: 3d coordinates.
        :type co: typing.Tuple[float]
        :param n: Number of points to find.
        :type n: int
        :rtype: typing.List
        :return: `Vector`, index, distance).
        '''
        pass

    def find_range(self, co: typing.Tuple[float],
                   radius: float) -> typing.List:
        ''' Find all points within ``radius`` of ``co``.

        :param co: 3d coordinates.
        :type co: typing.Tuple[float]
        :param radius: Distance to search for points.
        :type radius: float
        :rtype: typing.List
        :return: `Vector`, index, distance).
        '''
        pass

    def insert(self, co: typing.Tuple[float], index: int):
        ''' Insert a point into the KDTree.

        :param co: Point 3d position.
        :type co: typing.Tuple[float]
        :param index: The index of the point.
        :type index: int
        '''
        pass

    def __init__(self, size):
        ''' 

        '''
        pass



import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def load_image(imagepath,
               dirname: str = '',
               place_holder: bool = False,
               recursive: bool = False,
               ncase_cmp: bool = True,
               convert_callback=None,
               verbose=False,
               relpath: str = None,
               check_existing: bool = False,
               force_reload: bool = False) -> 'bpy.types.Image':
    ''' Return an image from the file path with options to search multiple paths and return a placeholder if its not found.

    :param filepath: The image filename If a path precedes it, this will be searched as well.
    :type filepath: str
    :param dirname: is the directory where the image may be located - any file at the end will be ignored.
    :type dirname: str
    :param place_holder: if True a new place holder image will be created. this is useful so later you can relink the image to its original data.
    :type place_holder: bool
    :param recursive: If True, directories will be recursively searched. Be careful with this if you have files in your root directory because it may take a long time.
    :type recursive: bool
    :param ncase_cmp: on non windows systems, find the correct case for the file.
    :type ncase_cmp: bool
    :param convert_callback: a function that takes an existing path and returns a new one. Use this when loading image formats blender may not support, the CONVERT_CALLBACK can take the path for a GIF (for example), convert it to a PNG and return the PNG's path. For formats blender can read, simply return the path that is given.
    :param relpath: If not None, make the file relative to this path.
    :type relpath: str
    :param check_existing: If true, returns already loaded image datablock if possible (based on file path).
    :type check_existing: bool
    :param force_reload: If true, force reloading of image (only useful when `check_existing` is also enabled).
    :type force_reload: bool
    :rtype: 'bpy.types.Image'
    :return: an image or None
    '''

    pass



import sys
import typing
import mathutils

GenericType = typing.TypeVar("GenericType")


def area_tri(v1: 'mathutils.Vector', v2: 'mathutils.Vector',
             v3: 'mathutils.Vector'):
    ''' Returns the area size of the 2D or 3D triangle defined.

    :param v1: Point1
    :type v1: 'mathutils.Vector'
    :param v2: Point2
    :type v2: 'mathutils.Vector'
    :param v3: Point3
    :type v3: 'mathutils.Vector'
    '''

    pass


def barycentric_transform(
        point: 'mathutils.Vector', tri_a1: 'mathutils.Vector',
        tri_a2: 'mathutils.Vector', tri_a3: 'mathutils.Vector',
        tri_b1: 'mathutils.Vector', tri_b2: 'mathutils.Vector',
        tri_b3: 'mathutils.Vector'):
    ''' Return a transformed point, the transformation is defined by 2 triangles.

    :param point: The point to transform.
    :type point: 'mathutils.Vector'
    :param tri_a1: source triangle vertex.
    :type tri_a1: 'mathutils.Vector'
    :param tri_a2: source triangle vertex.
    :type tri_a2: 'mathutils.Vector'
    :param tri_a3: source triangle vertex.
    :type tri_a3: 'mathutils.Vector'
    :param tri_b1: target triangle vertex.
    :type tri_b1: 'mathutils.Vector'
    :param tri_b2: target triangle vertex.
    :type tri_b2: 'mathutils.Vector'
    :param tri_b3: target triangle vertex.
    :type tri_b3: 'mathutils.Vector'
    :return: The transformed point
    '''

    pass


def box_fit_2d(points: typing.List) -> float:
    ''' Returns an angle that best fits the points to an axis aligned rectangle

    :param points: list of 2d points.
    :type points: typing.List
    :rtype: float
    :return: angle
    '''

    pass


def box_pack_2d(boxes: typing.List) -> typing.Tuple:
    ''' Returns a tuple with the width and height of the packed bounding box.

    :param boxes: list of boxes, each box is a list where the first 4 items are [x, y, width, height, ...] other items are ignored.
    :type boxes: typing.List
    :rtype: typing.Tuple
    :return: the width and height of the packed bounding box
    '''

    pass


def closest_point_on_tri(pt: 'mathutils.Vector', tri_p1: 'mathutils.Vector',
                         tri_p2: 'mathutils.Vector',
                         tri_p3: 'mathutils.Vector') -> 'mathutils.Vector':
    ''' Takes 4 vectors: one is the point and the next 3 define the triangle.

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param tri_p1: First point of the triangle
    :type tri_p1: 'mathutils.Vector'
    :param tri_p2: Second point of the triangle
    :type tri_p2: 'mathutils.Vector'
    :param tri_p3: Third point of the triangle
    :type tri_p3: 'mathutils.Vector'
    :rtype: 'mathutils.Vector'
    :return: The closest point of the triangle.
    '''

    pass


def convex_hull_2d(points: typing.List) -> typing.List[int]:
    ''' Returns a list of indices into the list given

    :param points: list of 2d points.
    :type points: typing.List
    :rtype: typing.List[int]
    :return: a list of indices
    '''

    pass


def delaunay_2d_cdt(
        vert_coords: typing.List['mathutils.Vector'],
        edges: typing.List[int],
        faces: typing.List[int],
        output_type,
        epsilon: float,
        need_ids=True
) -> typing.Union[typing.List[int], typing.List['mathutils.Vector']]:
    ''' Computes the Constrained Delaunay Triangulation of a set of vertices, with edges and faces that must appear in the triangulation. Some triangles may be eaten away, or combined with other triangles, according to output type. The returned verts may be in a different order from input verts, may be moved slightly, and may be merged with other nearby verts. The three returned orig lists give, for each of verts, edges, and faces, the list of input element indices corresponding to the positionally same output element. For edges, the orig indices start with the input edges and then continue with the edges implied by each of the faces (n of them for an n-gon). If the need_ids argument is supplied, and False, then the code skips the preparation of the orig arrays, which may save some time.

    :param vert_coords: Vertex coordinates (2d)
    :type vert_coords: typing.List['mathutils.Vector']
    :param edges: Edges, as pairs of indices in `vert_coords`
    :type edges: typing.List[int]
    :param faces: Faces, each sublist is a face, as indices in `vert_coords` (CCW oriented)
    :type faces: typing.List[int]
    :param output_type: What output looks like. 0 => triangles with convex hull. 1 => triangles inside constraints. 2 => the input constraints, intersected. 3 => like 2 but detect holes and omit them from output. 4 => like 2 but with extra edges to make valid BMesh faces. 5 => like 4 but detect holes and omit them from output.
    :type epsilon: float
    :type need_args: bool
    :param need_ids:  are the orig output arrays needed?
    :rtype: typing.Union[typing.List[int], typing.List['mathutils.Vector']]
    :return: Output tuple, (vert_coords, edges, faces, orig_verts, orig_edges, orig_faces)
    '''

    pass


def distance_point_to_plane(pt: 'mathutils.Vector',
                            plane_co: 'mathutils.Vector',
                            plane_no: 'mathutils.Vector'):
    ''' Returns the signed distance between a point and a plane (negative when below the normal).

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param plane_co: A point on the plane
    :type plane_co: 'mathutils.Vector'
    :param plane_no: The direction the plane is facing
    :type plane_no: 'mathutils.Vector'
    '''

    pass


def interpolate_bezier(knot1: 'mathutils.Vector', handle1: 'mathutils.Vector',
                       handle2: 'mathutils.Vector', knot2: 'mathutils.Vector',
                       resolution: int) -> typing.List:
    ''' Interpolate a bezier spline segment.

    :param knot1: First bezier spline point.
    :type knot1: 'mathutils.Vector'
    :param handle1: First bezier spline handle.
    :type handle1: 'mathutils.Vector'
    :param handle2: Second bezier spline handle.
    :type handle2: 'mathutils.Vector'
    :param knot2: Second bezier spline point.
    :type knot2: 'mathutils.Vector'
    :param resolution: Number of points to return.
    :type resolution: int
    :rtype: typing.List
    :return: The interpolated points
    '''

    pass


def intersect_line_line(v1: 'mathutils.Vector', v2: 'mathutils.Vector',
                        v3: 'mathutils.Vector', v4: 'mathutils.Vector'):
    ''' Returns a tuple with the points on each line respectively closest to the other.

    :param v1: First point of the first line
    :type v1: 'mathutils.Vector'
    :param v2: Second point of the first line
    :type v2: 'mathutils.Vector'
    :param v3: First point of the second line
    :type v3: 'mathutils.Vector'
    :param v4: Second point of the second line
    :type v4: 'mathutils.Vector'
    '''

    pass


def intersect_line_line_2d(lineA_p1: 'mathutils.Vector',
                           lineA_p2: 'mathutils.Vector',
                           lineB_p1: 'mathutils.Vector',
                           lineB_p2: 'mathutils.Vector') -> 'mathutils.Vector':
    ''' Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None.

    :param lineA_p1: First point of the first line
    :type lineA_p1: 'mathutils.Vector'
    :param lineA_p2: Second point of the first line
    :type lineA_p2: 'mathutils.Vector'
    :param lineB_p1: First point of the second line
    :type lineB_p1: 'mathutils.Vector'
    :param lineB_p2: Second point of the second line
    :type lineB_p2: 'mathutils.Vector'
    :rtype: 'mathutils.Vector'
    :return: The point of intersection or None when not found
    '''

    pass


def intersect_line_plane(line_a: 'mathutils.Vector',
                         line_b: 'mathutils.Vector',
                         plane_co: 'mathutils.Vector',
                         plane_no: 'mathutils.Vector',
                         no_flip=False) -> 'mathutils.Vector':
    ''' Calculate the intersection between a line (as 2 vectors) and a plane. Returns a vector for the intersection or None.

    :param line_a: First point of the first line
    :type line_a: 'mathutils.Vector'
    :param line_b: Second point of the first line
    :type line_b: 'mathutils.Vector'
    :param plane_co: A point on the plane
    :type plane_co: 'mathutils.Vector'
    :param plane_no: The direction the plane is facing
    :type plane_no: 'mathutils.Vector'
    :rtype: 'mathutils.Vector'
    :return: The point of intersection or None when not found
    '''

    pass


def intersect_line_sphere(line_a: 'mathutils.Vector',
                          line_b: 'mathutils.Vector',
                          sphere_co: 'mathutils.Vector',
                          sphere_radius,
                          clip=True) -> typing.Tuple:
    ''' Takes a line (as 2 points) and a sphere (as a point and a radius) and returns the intersection

    :param line_a: First point of the line
    :type line_a: 'mathutils.Vector'
    :param line_b: Second point of the line
    :type line_b: 'mathutils.Vector'
    :param sphere_co: The center of the sphere
    :type sphere_co: 'mathutils.Vector'
    :param sphere_radius: Radius of the sphere
    :rtype: typing.Tuple
    :return: The intersection points as a pair of vectors or None when there is no intersection
    '''

    pass


def intersect_line_sphere_2d(line_a: 'mathutils.Vector',
                             line_b: 'mathutils.Vector',
                             sphere_co: 'mathutils.Vector',
                             sphere_radius,
                             clip=True) -> typing.Tuple:
    ''' Takes a line (as 2 points) and a sphere (as a point and a radius) and returns the intersection

    :param line_a: First point of the line
    :type line_a: 'mathutils.Vector'
    :param line_b: Second point of the line
    :type line_b: 'mathutils.Vector'
    :param sphere_co: The center of the sphere
    :type sphere_co: 'mathutils.Vector'
    :param sphere_radius: Radius of the sphere
    :rtype: typing.Tuple
    :return: The intersection points as a pair of vectors or None when there is no intersection
    '''

    pass


def intersect_plane_plane(plane_a_co: 'mathutils.Vector',
                          plane_a_no: 'mathutils.Vector',
                          plane_b_co: 'mathutils.Vector',
                          plane_b_no: 'mathutils.Vector') -> typing.Tuple:
    ''' Return the intersection between two planes

    :param plane_a_co: Point on the first plane
    :type plane_a_co: 'mathutils.Vector'
    :param plane_a_no: Normal of the first plane
    :type plane_a_no: 'mathutils.Vector'
    :param plane_b_co: Point on the second plane
    :type plane_b_co: 'mathutils.Vector'
    :param plane_b_no: Normal of the second plane
    :type plane_b_no: 'mathutils.Vector'
    :rtype: typing.Tuple
    :return: The line of the intersection represented as a point and a vector
    '''

    pass


def intersect_point_line(pt: 'mathutils.Vector', line_p1: 'mathutils.Vector',
                         line_p2):
    ''' Takes a point and a line and returns a tuple with the closest point on the line and its distance from the first point of the line as a percentage of the length of the line.

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param line_p1: First point of the line Second point of the line
    :type line_p1: 'mathutils.Vector'
    :param line_p1: First point of the line Second point of the line
    :type line_p1: 'mathutils.Vector'
    '''

    pass


def intersect_point_quad_2d(
        pt: 'mathutils.Vector', quad_p1: 'mathutils.Vector',
        quad_p2: 'mathutils.Vector', quad_p3: 'mathutils.Vector',
        quad_p4: 'mathutils.Vector'):
    ''' Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad, only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0. Works only with convex quads without singular edges.

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param quad_p1: First point of the quad
    :type quad_p1: 'mathutils.Vector'
    :param quad_p2: Second point of the quad
    :type quad_p2: 'mathutils.Vector'
    :param quad_p3: Third point of the quad
    :type quad_p3: 'mathutils.Vector'
    :param quad_p4: Fourth point of the quad
    :type quad_p4: 'mathutils.Vector'
    '''

    pass


def intersect_point_tri(pt: 'mathutils.Vector', tri_p1: 'mathutils.Vector',
                        tri_p2: 'mathutils.Vector',
                        tri_p3: 'mathutils.Vector') -> 'mathutils.Vector':
    ''' Takes 4 vectors: one is the point and the next 3 define the triangle. Projects the point onto the triangle plane and checks if it is within the triangle.

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param tri_p1: First point of the triangle
    :type tri_p1: 'mathutils.Vector'
    :param tri_p2: Second point of the triangle
    :type tri_p2: 'mathutils.Vector'
    :param tri_p3: Third point of the triangle
    :type tri_p3: 'mathutils.Vector'
    :rtype: 'mathutils.Vector'
    :return: Point on the triangles plane or None if its outside the triangle
    '''

    pass


def intersect_point_tri_2d(pt: 'mathutils.Vector', tri_p1: 'mathutils.Vector',
                           tri_p2: 'mathutils.Vector',
                           tri_p3: 'mathutils.Vector'):
    ''' Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns 1 if the point is within the triangle, otherwise 0.

    :param pt: Point
    :type pt: 'mathutils.Vector'
    :param tri_p1: First point of the triangle
    :type tri_p1: 'mathutils.Vector'
    :param tri_p2: Second point of the triangle
    :type tri_p2: 'mathutils.Vector'
    :param tri_p3: Third point of the triangle
    :type tri_p3: 'mathutils.Vector'
    '''

    pass


def intersect_ray_tri(v1: 'mathutils.Vector',
                      v2: 'mathutils.Vector',
                      v3: 'mathutils.Vector',
                      ray: 'mathutils.Vector',
                      orig: 'mathutils.Vector',
                      clip: bool = True) -> 'mathutils.Vector':
    ''' Returns the intersection between a ray and a triangle, if possible, returns None otherwise.

    :param v1: Point1
    :type v1: 'mathutils.Vector'
    :param v2: Point2
    :type v2: 'mathutils.Vector'
    :param v3: Point3
    :type v3: 'mathutils.Vector'
    :param ray: Direction of the projection
    :type ray: 'mathutils.Vector'
    :param orig: Origin
    :type orig: 'mathutils.Vector'
    :param clip: When False, don't restrict the intersection to the area of the triangle, use the infinite plane defined by the triangle.
    :type clip: bool
    :rtype: 'mathutils.Vector'
    :return: The point of intersection or None if no intersection is found
    '''

    pass


def intersect_sphere_sphere_2d(p_a: 'mathutils.Vector', radius_a: float,
                               p_b: 'mathutils.Vector', radius_b: float):
    ''' Returns 2 points on between intersecting circles.

    :param p_a: Center of the first circle
    :type p_a: 'mathutils.Vector'
    :param radius_a: Radius of the first circle
    :type radius_a: float
    :param p_b: Center of the second circle
    :type p_b: 'mathutils.Vector'
    :param radius_b: Radius of the second circle
    :type radius_b: float
    '''

    pass


def intersect_tri_tri_2d(tri_a1, tri_a2, tri_a3, tri_b1, tri_b2, tri_b3):
    ''' Check if two 2D triangles intersect.

    '''

    pass


def normal(vectors: typing.List):
    ''' Returns the normal of a 3D polygon.

    :param vectors: Vectors to calculate normals with
    :type vectors: typing.List
    '''

    pass


def points_in_planes(planes: typing.List['mathutils.Vector']):
    ''' Returns a list of points inside all planes given and a list of index values for the planes used.

    :param planes: List of planes (4D vectors).
    :type planes: typing.List['mathutils.Vector']
    :return: two lists, once containing the vertices inside the planes, another containing the plane indices used
    '''

    pass


def tessellate_polygon(veclist_list):
    ''' Takes a list of polylines (each point a pair or triplet of numbers) and returns the point indices for a polyline filled with triangles. Does not handle degenerate geometry (such as zero-length lines due to consecutive identical points).

    :param veclist_list:  list of polylines
    '''

    pass


def volume_tetrahedron(v1: 'mathutils.Vector', v2: 'mathutils.Vector',
                       v3: 'mathutils.Vector', v4: 'mathutils.Vector'):
    ''' Return the volume formed by a tetrahedron (points can be in any order).

    :param v1: Point1
    :type v1: 'mathutils.Vector'
    :param v2: Point2
    :type v2: 'mathutils.Vector'
    :param v3: Point3
    :type v3: 'mathutils.Vector'
    :param v4: Point4
    :type v4: 'mathutils.Vector'
    '''

    pass



import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def svg(override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: bool = None,
        *,
        filepath: str = "",
        filter_glob: str = "*.svg"):
    ''' Load a SVG file

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: bool
    :param filepath: File Path, Filepath used for importing the file
    :type filepath: str
    :param filter_glob: filter_glob
    :type filter_glob: str
    '''

    pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


def poly_3d_calc(veclist, pt):
    ''' Calculate barycentric weights for a point on a polygon.

    :param veclist:  list of vectors
    :param pt:  list of per-vector weights
    '''

    pass



import sys
import typing
import bpy.types
import freestyle.types

from . import ContextFunctions

GenericType = typing.TypeVar("GenericType")


class BoundingBox:
    ''' Object representing a bounding box consisting out of 2 2D vectors
    '''

    def inside(self, other):
        ''' True if self inside other, False otherwise

        '''
        pass


class StrokeCollector:
    ''' Collects and Stores stroke objects
    '''

    def shade(self, stroke):
        ''' 

        '''
        pass


def angle_x_normal(i):
    ''' unsigned angle between a Point's normal and the X axis, in radians

    '''

    pass


def bound(lower, x, higher):
    ''' Returns x bounded by a maximum and minimum value. Equivalent to: return min(max(x, lower), higher)

    '''

    pass


def bounding_box(stroke):
    ''' Returns the maximum and minimum coordinates (the bounding box) of the stroke's vertices

    '''

    pass


def curvature_from_stroke_vertex(svert):
    ''' The 3D curvature of an stroke vertex' underlying geometry The result is None or in the range [-inf, inf]

    '''

    pass


def find_matching_vertex(id, it):
    ''' Finds the matching vertex, or returns None.

    '''

    pass


def getCurrentScene() -> 'bpy.types.Scene':
    ''' Returns the current scene.

    :rtype: 'bpy.types.Scene'
    :return: The current scene.
    '''

    pass


def get_chain_length(ve, orientation):
    ''' Returns the 2d length of a given ViewEdge.

    '''

    pass


def get_object_name(stroke):
    ''' Returns the name of the object that this stroke is drawn on.

    '''

    pass


def get_strokes():
    ''' Get all strokes that are currently available

    '''

    pass


def get_test_stroke():
    ''' Returns a static stroke object for testing

    '''

    pass


def integrate(func: 'freestyle.types.UnaryFunction0D',
              it: 'freestyle.types.Interface0DIterator',
              it_end: 'freestyle.types.Interface0DIterator',
              integration_type: 'freestyle.types.IntegrationType'
              ) -> typing.Union[int, float]:
    ''' Returns a single value from a set of values evaluated at each 0D element of this 1D element.

    :param func: The UnaryFunction0D used to compute a value at each Interface0D.
    :type func: 'freestyle.types.UnaryFunction0D'
    :param it: The Interface0DIterator used to iterate over the 0D elements of this 1D element. The integration will occur over the 0D elements starting from the one pointed by it.
    :type it: 'freestyle.types.Interface0DIterator'
    :param it_end: The Interface0DIterator pointing the end of the 0D elements of the 1D element.
    :type it_end: 'freestyle.types.Interface0DIterator'
    :param integration_type: The integration method used to compute a single value from a set of values.
    :type integration_type: 'freestyle.types.IntegrationType'
    :rtype: typing.Union[int, float]
    :return: The single value obtained for the 1D element. The return value type is float if func is of the `UnaryFunction0DDouble` or `UnaryFunction0DFloat` type, and int if func is of the `UnaryFunction0DUnsigned` type.
    '''

    pass


def is_poly_clockwise(stroke):
    ''' True if the stroke is orientated in a clockwise way, False otherwise

    '''

    pass


def iter_distance_along_stroke(stroke):
    ''' Yields the absolute distance along the stroke up to the current vertex.

    '''

    pass


def iter_distance_from_camera(stroke, range_min, range_max, normfac):
    ''' Yields the distance to the camera relative to the maximum possible distance for every stroke vertex, constrained by given minimum and maximum values.

    '''

    pass


def iter_distance_from_object(stroke, location, range_min, range_max, normfac):
    ''' yields the distance to the given object relative to the maximum possible distance for every stroke vertex, constrained by given minimum and maximum values.

    '''

    pass


def iter_material_value(stroke, func, attribute):
    ''' Yields a specific material attribute from the vertex' underlying material.

    '''

    pass


def iter_t2d_along_stroke(stroke):
    ''' Yields the progress along the stroke.

    '''

    pass


def material_from_fedge(fe):
    ''' get the diffuse rgba color from an FEdge

    '''

    pass


def normal_at_I0D(i):
    ''' Normal at an Interface0D object. In contrast to Normal2DF0D this function uses the actual data instead of underlying Fedge objects.

    '''

    pass


def pairwise(iterable, types={}):
    ''' Yields a tuple containing the previous and current object

    '''

    pass


def rgb_to_bw(r, g, b):
    ''' Method to convert rgb to a bw intensity value.

    '''

    pass


def simplify(points, tolerance):
    ''' Simplifies a set of points

    '''

    pass


def stroke_curvature(it):
    ''' Compute the 2D curvature at the stroke vertex pointed by the iterator 'it'. K = 1 / R where R is the radius of the circle going through the current vertex and its neighbors

    '''

    pass


def stroke_normal(stroke):
    ''' Compute the 2D normal at the stroke vertex pointed by the iterator 'it'. It is noted that Normal2DF0D computes normals based on underlying FEdges instead, which is inappropriate for strokes when they have already been modified by stroke geometry modifiers. The returned normals are dynamic: they update when the vertex position (and therefore the vertex normal) changes. for use in geometry modifiers it is advised to cast this generator function to a tuple or list

    '''

    pass


def tripplewise(iterable):
    ''' Yields a tuple containing the current object and its immediate neighbors

    '''

    pass
